//
//  ASIParseMarkup.m
//  Marker For IOS
//
//  Created by Alex Jurgensen on 2012-12-18.
//  Copyright (c) 2012 ASInnovations. All rights reserved.
//

#import "ASIParseMarkup.h"

@implementation ASIParseMarkup

-( void )parseMarkupFromFile:( NSString* )file withOwner:(id)owner
{
    NSString *path = [[NSBundle mainBundle] pathForResource:[self getDeviceSpecificMarkupFile:file] ofType:@"ASIMarkup"];
    NSDictionary *markup = [NSJSONSerialization JSONObjectWithData:[NSData dataWithContentsOfFile:path] options:kNilOptions error:nil];
    [self parseObjectForMarkup:[markup objectForKey:@"UIObjects"] withParent:nil andOwner:owner];
}

-( void )parseObjectForMarkup:( NSArray* )object withParent:( id )parent andOwner:( id ) owner
{
    for ( NSDictionary *element in object )
    {
        NSLog(@"%@", [element objectForKey:@"kind"]);
        id theObject = [self getObjectOfClass:[element objectForKey:@"kind"] withOptions:element];
        if (!parent)
        {
            [owner setView:theObject];
        }
        
        if ( [element objectForKey:@"children"] )
        {
            if (!parent) {
                    [self parseObjectForMarkup:[element objectForKey:@"children"] withParent:[owner view] andOwner:owner];
                } else {
                    [self parseObjectForMarkup:[element objectForKey:@"children"] withParent:theObject andOwner:owner];
                }
        }
        
        if (parent)
        {
            NSLog(@"Adding Parent");
            [parent addSubview:theObject];
            [self makeConnectionForObject:theObject toOutletWithId:[element objectForKey:@"id"] andOwner:owner];
        }
    }
}

-( id )getObjectOfClass:( NSString* )class withOptions:( NSDictionary* )options
{
    Class objectClass = NSClassFromString(class);
    id object = [[objectClass alloc] initWithMarkup:options];
    if ( [[options objectForKey:@"type"] isEqualToString:@"controller"] )
    {
        return [object view];
    } else {
        return object;
    }
}

-( NSString* )getDeviceSpecificMarkupFile:( NSString* )fileName
{
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    {
        return [NSString stringWithFormat:@"%@-iPad",fileName];
    }
    else{
        return fileName;
}
}

-( void )makeConnectionForObject:( id )object toOutletWithId:( NSString* )outletId andOwner:( id )owner
{
    if ( outletId )
    {
        SEL variables = NSSelectorFromString([NSString stringWithFormat:@"set%@:",outletId] );
        if ( [owner respondsToSelector:variables] )
        {
            [owner performSelector:variables withObject:object];
        }
}
}

@end
